# -*- coding: utf-8 -*-
"""
/***************************************************************************
 CADDigitize
                                 A QGIS plugin
 CAD like tools for QGis
 Fork of Rectangles Ovals Digitizing. Inspired by CadTools, LibreCAD/AutoCAD.
                              -------------------
        begin                : 2014-08-11
        git sha              : $Format:%H$
        copyright            : (C) 2014 by Loïc BARTOLETTI
        email                : l.bartoletti@free.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""

# List comprehensions in canvasMoveEvent functions are
# adapted from Benjamin Bohard`s part of rectovaldiams plugin.

from PyQt4.QtCore import *
from PyQt4.QtGui import *
from qgis.core import *
from qgis.gui import *
from qgis.utils import iface
from math import *
from tools.rubber import *
from tools.snap import *
from tools.calc import *
from tools.rectangle import *

class ToolBar:
    #####################
    #       Rect        #
    #####################
    def __init__(self):
        self.optionsToolBar = iface.mainWindow().findChild(
                QToolBar, u"CADDigitize Options")
        self.clear()

#   No options needed
#    def rectOptions(self):
#        self.optionsToolBar.clear()

    def clear(self):
        self.optionsToolBar.clear()


class RectBy3PointsTool(QgsMapTool):
    def __init__(self, canvas):
        QgsMapTool.__init__(self,canvas)
        self.canvas = canvas
        self.nbPoints = 0
        self.rb = None
        self.rb_snap = None
        self.x_p1, self.y_p1, self.x_p2, self.y_p2, self.x_p3, self.y_p3, self.x_p4, self.y_p4 = None, None, None, None, None, None, None, None
        self.length = 0
        #our own fancy cursor
        self.cursor = QCursor(QPixmap(["16 16 3 1",
                                      "      c None",
                                      ".     c #FF0000",
                                      "+     c #1210f3",
                                      "                ",
                                      "       +.+      ",
                                      "      ++.++     ",
                                      "     +.....+    ",
                                      "    +.     .+   ",
                                      "   +.   .   .+  ",
                                      "  +.    .    .+ ",
                                      " ++.    .    .++",
                                      " ... ...+... ...",
                                      " ++.    .    .++",
                                      "  +.    .    .+ ",
                                      "   +.   .   .+  ",
                                      "   ++.     .+   ",
                                      "    ++.....+    ",
                                      "      ++.++     ",
                                      "       +.+      "]))

    def clear(self):
        self.nbPoints = 0
        self.x_p1, self.y_p1, self.x_p2, self.y_p2, self.x_p3, self.y_p3 = None, None, None, None, None, None
        if self.rb_snap:
            self.rb_snap.reset(True)
        self.rb_snap=None
        if self.rb:
            self.rb.reset(True)
        self.rb=None

    def keyReleaseEvent(self,  event):
        if event.key() == Qt.Key_Escape:
            self.clear()
            self.canvas.refresh()

            return

    def calcPoint(p):
        return p.x() + self.length * cos(radians(90) + self.angle_exist), self.p.y() + self.length * sin(radians(90) + self.angle_exist)

    def canvasPressEvent(self,event):
        layer = self.canvas.currentLayer()
        if self.nbPoints == 0:
            color = QColor(255,0,0)
            self.rb = myRubberDigit(self.canvas)
        elif self.nbPoints == 2:
            self.rb.reset(True)
            self.rb=None

            self.canvas.refresh()

        snapped, pointMap = snappedPoint(self.canvas, event)
        if self.nbPoints == 0:
            self.x_p1 = pointMap.x()
            self.y_p1 = pointMap.y()
        elif self.nbPoints == 1:
            self.x_p2 = pointMap.x()
            self.y_p2 = pointMap.y()
        else:
            self.x_p3 = pointMap.x()
            self.y_p3 = pointMap.y()


        self.nbPoints += 1

        if self.nbPoints == 3:
            geom = Rectangle.getRectBy3Points(QgsPoint(self.x_p1, self.y_p1), QgsPoint(self.x_p2, self.y_p2), QgsPoint(self.x_p3, self.y_p3))

            self.nbPoints = 0
            self.x_p1, self.y_p1, self.x_p2, self.y_p2, self.x_p3, self.y_p3 = None, None, None, None, None, None

            self.emit(SIGNAL("rbFinished(PyQt_PyObject)"), geom)
        if self.rb_snap:
            self.rb_snap.reset(True)
        if self.rb:return

    def canvasMoveEvent(self,event):
        if self.rb_snap:
            self.rb_snap.reset(True)

        snapped, point = snappedPoint(self.canvas, event)
        if snapped:
            self.rb_snap = myRubberPointSnap(self.canvas)
            self.rb_snap.setToGeometry(QgsGeometry().fromPoint(point), None)

        if self.nbPoints == 1:
            self.rb.setToGeometry(QgsGeometry.fromPolyline([QgsPoint(self.x_p1, self.y_p1), point]), None)
        if self.nbPoints >= 2:
            geom = Rectangle.getRectBy3Points(QgsPoint(self.x_p1, self.y_p1), QgsPoint(self.x_p2, self.y_p2), point)
            self.rb.setToGeometry(geom, None)

    def showSettingsWarning(self):
        pass

    def activate(self):
        self.canvas.setCursor(self.cursor)
        self.optionsToolbar = ToolBar()

    def deactivate(self):
        self.clear()
        self.optionsToolbar.clear()
        self.canvas.refresh()

    def isZoomTool(self):
        return False

    def isTransient(self):
        return False

    def isEditTool(self):
        return True


class RectByExtentTool(QgsMapTool):
    def __init__(self, canvas):
        QgsMapTool.__init__(self,canvas)
        self.canvas=canvas
        self.nbPoints = 0
        self.rb=None
        self.rb_snap=None
        self.x_p1, self.y_p1, self.x_p2, self.y_p2 = None, None, None, None
        #our own fancy cursor
        self.cursor = QCursor(QPixmap(["16 16 3 1",
                                       "      c None",
                                       ".     c #FF0000",
                                       "+     c #17a51a",
                                       "                ",
                                       "       +.+      ",
                                       "      ++.++     ",
                                       "     +.....+    ",
                                       "    +.  .  .+   ",
                                       "   +.   .   .+  ",
                                       "  +.    .    .+ ",
                                       " ++.    .    .++",
                                       " ... ...+... ...",
                                       " ++.    .    .++",
                                       "  +.    .    .+ ",
                                       "   +.   .   .+  ",
                                       "   ++.  .  .+   ",
                                       "    ++.....+    ",
                                       "      ++.++     ",
                                       "       +.+      "]))


    def clear(self):
        self.nbPoints = 0
        self.x_p1, self.y_p1, self.x_p2, self.y_p2 = None, None, None, None
        if self.rb_snap:
            self.rb_snap.reset(True)
        self.rb_snap=None
        if self.rb:
            self.rb.reset(True)
        self.rb=None

    def keyReleaseEvent(self,  event):
        if event.key() == Qt.Key_Escape:
            self.clear()
            self.canvas.refresh()
            return

    def canvasPressEvent(self,event):
        layer = self.canvas.currentLayer()
        if self.nbPoints == 0:
	    	self.rb = myRubberDigit(self.canvas)
        else:
            self.rb.reset(True)
            self.rb=None

            self.canvas.refresh()

        snapped, pointMap = snappedPoint(self.canvas, event)

        if self.nbPoints == 0:
            self.x_p1 = pointMap.x()
            self.y_p1 = pointMap.y()
        else:
            self.x_p2 = pointMap.x()
            self.y_p2 = pointMap.y()

        self.nbPoints += 1

        if self.nbPoints == 2:
            geom = Rectangle.getRectByExtent(QgsPoint(self.x_p1, self.y_p1), QgsPoint(self.x_p2, self.y_p2))

            self.nbPoints = 0
            self.x_p1, self.y_p1, self.x_p2, self.y_p2 = None, None, None, None

            self.emit(SIGNAL("rbFinished(PyQt_PyObject)"), geom)
        if self.rb_snap:
            self.rb_snap.reset(True)
        if self.rb:return


    def canvasMoveEvent(self,event):
        if self.rb_snap:
            self.rb_snap.reset(True)

        snapped, point = snappedPoint(self.canvas, event)
        if snapped:
            self.rb_snap = myRubberPointSnap(self.canvas)
            self.rb_snap.setToGeometry(QgsGeometry().fromPoint(point), None)

        if self.rb:
            geom = Rectangle.getRectByExtent(QgsPoint(self.x_p1, self.y_p1), point)
            self.rb.setToGeometry(geom, None)

    def showSettingsWarning(self):
        pass

    def activate(self):
        self.canvas.setCursor(self.cursor)
        self.optionsToolbar = ToolBar()

    def deactivate(self):
        self.clear()
        self.optionsToolbar.clear()

        self.canvas.refresh()

    def isZoomTool(self):
        return False

    def isTransient(self):
        return False

    def isEditTool(self):
        return True

# Tool class
class RectFromCenterTool(QgsMapTool):
    def __init__(self, canvas):
        QgsMapTool.__init__(self,canvas)
        self.canvas=canvas
        self.nbPoints = 0
        self.rb=None
        self.rb_snap=None
        self.xc, self.yc, self.x_p2, self.y_p2 = None, None, None, None
        #our own fancy cursor
        self.cursor = QCursor(QPixmap(["16 16 3 1",
                                       "      c None",
                                       ".     c #FF0000",
                                       "+     c #17a51a",
                                       "                ",
                                       "       +.+      ",
                                       "      ++.++     ",
                                       "     +.....+    ",
                                       "    +.  .  .+   ",
                                       "   +.   .   .+  ",
                                       "  +.    .    .+ ",
                                       " ++.    .    .++",
                                       " ... ...+... ...",
                                       " ++.    .    .++",
                                       "  +.    .    .+ ",
                                       "   +.   .   .+  ",
                                       "   ++.  .  .+   ",
                                       "    ++.....+    ",
                                       "      ++.++     ",
                                       "       +.+      "]))


    def clear(self):
        self.nbPoints = 0
        self.xc, self.yc, self.x_p2, self.y_p2 = None, None, None, None
        if self.rb_snap:
            self.rb_snap.reset(True)
        self.rb_snap=None
        if self.rb:
            self.rb.reset(True)
        self.rb=None

    def keyReleaseEvent(self,  event):
        if event.key() == Qt.Key_Escape:
            self.clear()
            self.canvas.refresh()
            return

    def canvasPressEvent(self,event):
        layer = self.canvas.currentLayer()
        if self.nbPoints == 0:
	    	self.rb = myRubberDigit(self.canvas)
        else:
            self.rb.reset(True)
            self.rb=None

            self.canvas.refresh()
        snapped, pointMap = snappedPoint(self.canvas, event)

        if self.nbPoints == 0:
            self.xc = pointMap.x()
            self.yc = pointMap.y()
        else:
            self.x_p2 = pointMap.x()
            self.y_p2 = pointMap.y()

        self.nbPoints += 1

        if self.nbPoints == 2:

            geom = Rectangle.getRectFromCenter(QgsPoint(self.xc, self.yc), QgsPoint(self.x_p2, self.y_p2))

            self.nbPoints = 0
            self.xc, self.yc, self.x_p2, self.y_p2 = None, None, None, None

            self.emit(SIGNAL("rbFinished(PyQt_PyObject)"), geom)
        if self.rb_snap:
            self.rb_snap.reset(True)
        if self.rb:return

    def canvasMoveEvent(self,event):
        if self.rb_snap:
            self.rb_snap.reset(True)
        snapped, point = snappedPoint(self.canvas, event)
        if snapped:
            self.rb_snap = myRubberPointSnap(self.canvas)
            self.rb_snap.setToGeometry(QgsGeometry().fromPoint(point), None)

        if self.rb:
            geom = Rectangle.getRectFromCenter(QgsPoint(self.xc, self.yc), point)
            self.rb.setToGeometry(geom, None)

    def showSettingsWarning(self):
        pass

    def activate(self):
        self.canvas.setCursor(self.cursor)
        self.optionsToolbar = ToolBar()

    def deactivate(self):
        self.clear()
        self.optionsToolbar.clear()
        self.canvas.refresh()

    def isZoomTool(self):
        return False

    def isTransient(self):
        return False

    def isEditTool(self):
        return True

# Tool class
class SquareFromCenterTool(QgsMapTool):
    def __init__(self, canvas):
        QgsMapTool.__init__(self,canvas)
        self.canvas=canvas
        self.nbPoints = 0
        self.rb=None
        self.rb_snap=None
        self.xc, self.yc, self.x_p2, self.y_p2 = None, None, None, None
        #our own fancy cursor
        self.cursor = QCursor(QPixmap(["16 16 3 1",
                                       "      c None",
                                       ".     c #FF0000",
                                       "+     c #17a51a",
                                       "                ",
                                       "       +.+      ",
                                       "      ++.++     ",
                                       "     +.....+    ",
                                       "    +.  .  .+   ",
                                       "   +.   .   .+  ",
                                       "  +.    .    .+ ",
                                       " ++.    .    .++",
                                       " ... ...+... ...",
                                       " ++.    .    .++",
                                       "  +.    .    .+ ",
                                       "   +.   .   .+  ",
                                       "   ++.  .  .+   ",
                                       "    ++.....+    ",
                                       "      ++.++     ",
                                       "       +.+      "]))

    def clear(self):
        self.nbPoints = 0
        self.xc, self.yc, self.x_p2, self.y_p2 = None, None, None, None
        if self.rb_snap:
            self.rb_snap.reset(True)
        self.rb_snap=None
        if self.rb:
            self.rb.reset(True)
        self.rb=None

    def keyReleaseEvent(self,  event):
        if event.key() == Qt.Key_Escape:
            self.clear()
            self.canvas.refresh()
            return

    def canvasPressEvent(self,event):
        layer = self.canvas.currentLayer()
        if self.nbPoints == 0:
	    	self.rb = myRubberDigit(self.canvas)
        else:
            self.rb.reset(True)
            self.rb=None

            self.canvas.refresh()

        snapped, pointMap = snappedPoint(self.canvas, event)

        if self.nbPoints == 0:
            self.xc = pointMap.x()
            self.yc = pointMap.y()
        else:
            self.x_p2 = pointMap.x()
            self.y_p2 = pointMap.y()

        self.nbPoints += 1

        if self.nbPoints == 2:

            geom = Rectangle.getSquareFromCenter(QgsPoint(self.xc, self.yc), QgsPoint(self.x_p2, self.y_p2))

            self.nbPoints = 0
            self.xc, self.yc, self.x_p2, self.y_p2 = None, None, None, None

            self.emit(SIGNAL("rbFinished(PyQt_PyObject)"), geom)
        if self.rb_snap:
            self.rb_snap.reset(True)
        if self.rb:return


    def canvasMoveEvent(self,event):
        if self.rb_snap:
            self.rb_snap.reset(True)

        snapped, point = snappedPoint(self.canvas, event)
        if snapped:
            self.rb_snap = myRubberPointSnap(self.canvas)
            self.rb_snap.setToGeometry(QgsGeometry().fromPoint(point), None)

        if self.rb:
            geom = Rectangle.getSquareFromCenter(QgsPoint(self.xc, self.yc), point)
	    self.rb.setToGeometry(geom, None)

    def showSettingsWarning(self):
        pass

    def activate(self):
        self.canvas.setCursor(self.cursor)
        self.optionsToolbar = ToolBar()

    def deactivate(self):
        self.clear()
        self.optionsToolbar.clear()

        self.canvas.refresh()

    def isZoomTool(self):
        return False

    def isTransient(self):
        return False

    def isEditTool(self):
        return True


