# -*- coding: utf-8 -*-
"""
/***************************************************************************
PostgisQuery
A QGIS plugin
Editor to build and perform PostGIS SQL Queries and adds the resulting vector layer to MapCanvas
                             -------------------
begin                : 2009-03-06 
copyright            : (C) 2009 by Dr. Horst Duester / kappasys
email                : horst.duester@kappasys.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""
# Import the PyQt and QGIS libraries
from PyQt4.QtCore import * 
from PyQt4.QtGui import *
from qgis.core import *
import subprocess, os,  sys
import apicompat
# Initialize Qt resources from file resources.py
import resources
# Import the code for the dialog
from PostgisQueryDialog import PostgisQueryDialog

class PostgisQuery: 

  def __init__(self, iface):
    # Save reference to the QGIS interface
    self.iface = iface
    
#Initialise thetranslation environment    
    userPluginPath = QFileInfo(QgsApplication.qgisUserDbFilePath()).path()+"/python/plugins/postgisquery"  
    systemPluginPath = QgsApplication.prefixPath()+"/share/qgis/python/plugins/postgisquery"
    myLocaleName = QLocale.system().name()
    myLocale = myLocaleName[0:2]
    if QFileInfo(userPluginPath).exists():
      pluginPath = userPluginPath+"/i18n/PostgisQuery_"+myLocale+".qm"
    elif QFileInfo(systemPluginPath).exists():
      pluginPath = systemPluginPath+"/i18n/PostgisQuery_"+myLocale+".qm"

    self.localePath = pluginPath
    if QFileInfo(self.localePath).exists():
      self.translator = QTranslator()
      self.translator.load(self.localePath)
      
      if qVersion() > '4.3.3':        
        QCoreApplication.installTranslator(self.translator)
  
#@brief Initializes the plugin
#@param none
#@return  none
#@author Dr. Horst Duester
#@date 16. March 2009
#@version 1.0
#@todo  
  def initGui(self):  
    # Create action that will start plugin configuration
    self.action = QAction(QIcon(":/plugins/postgisquery/icon.png"), \
        "PostGIS SQL Editor", self.iface.mainWindow())
    # connect the action to the run method
    QObject.connect(self.action, SIGNAL("triggered()"), self.run) 
    
    # Add toolbar button and menu item
    self.iface.addToolBarIcon(self.action)
    
    if hasattr(self.iface,  "addPluginToDatabaseMenu"):
        self.iface.addPluginToDatabaseMenu("&Postgis SQL Editor", self.action)
    else:
        self.iface.addPluginToMenu("&Postgis SQL Editor", self.action)

    
#@brief unloads the plugin from QGIS
#@param none
#@return  none
#@author Dr. Horst Duester
#@date 16. March 2009
#@version 1.0
#@todo  
  def unload(self):
    # Remove the plugin menu item and icon
    if hasattr(self.iface,  "addPluginToDatabaseMenu"):
       self.iface.removePluginDatabaseMenu("&Postgis SQL Editor", self.action)
    else:
       self.iface.removePluginMenu("&Postgis SQL Editor", self.action)

    self.iface.removeToolBarIcon(self.action)


#@brief run method that performs all the real work
#@param none
#@return  none
#@author Dr. Horst Duester
#@date 16. March 2009
#@version 1.0
#@todo     
  def run(self): 
     # create and show the dialog 
     if qVersion() >= '4.5':        
        self.flags = Qt.WindowTitleHint | Qt.WindowSystemMenuHint | Qt.WindowCloseButtonHint | Qt.WindowMinimizeButtonHint | Qt.WindowMaximizeButtonHint  # QgisGui.ModalDialogFlags
     else:
        self.flags = Qt.WindowTitleHint | Qt.WindowSystemMenuHint | Qt.WindowMinimizeButtonHint | Qt.WindowMaximizeButtonHint  # QgisGui.ModalDialogFlags
        
     self.dlg = PostgisQueryDialog(self.iface.mainWindow(), self.flags)     
     settings = QSettings()
     
     self.postgisQueryTmpPath = settings.value(pystring("/Plugin-PostGISQuery/tmpPath"))
     self.dlg.txtEditSql.setText(pystring(settings.value("/Plugin-PostGISQuery/query")))
     
     if self.postgisQueryTmpPath == '':
       self.postgisQueryTmpPath = QDir.tempPath()
       
     self.dlg.lblTempPath.setText(pystring(self.postgisQueryTmpPath))     
       
     self.dlg.openFileDialog.connect(self.setTmpPath)       
     self.dlg.loadLayer.connect(self.showVLayer) 
     self.dlg.cmbServer.currentIndexChanged.connect(self.setDBServer)
     self.dlg.loadFile.connect(self.loadSqlFile)
     self.dlg.saveFile.connect(self.saveSqlFile)
     
     self.dlg.initGui()
     self.dlg.show()

     pass 
       
       
#@brief Setting the default server name
#@param String Server Name
#@return  none
#@author Dr. Horst Duester
#@date 16. March 2009
#@version 1.0
#@todo     
  def setDBServer(self, dbServerName):
     settings = QSettings()
     settings.setValue("PostgreSQL/connections/selected", pystring(dbServerName))  
     pass
     
     
#@brief Setting of the temporary path
#@param none
#@return  none
#@author Dr. Horst Duester
#@date 10. March 2009
#@version 1.0
#@todo
  def  setTmpPath(self):
     settings = QSettings()
#     self.postgisQueryTmpPath = QFileDialog(self.iface.mainWindow(), self.flags).getExistingDirectory(self.iface.mainWindow(),QCoreApplication.translate("PostgisQuery","Please select path to temporary directory"))
     self.postgisQueryTmpPath = QFileDialog.getExistingDirectory(None,QCoreApplication.translate("PostgisQuery","Please select path to temporary directory"))
     
     if self.postgisQueryTmpPath != "":
       settings.setValue("/Plugin-PostGISQuery/tmpPath", pystring(self.postgisQueryTmpPath))
       self.dlg.lblTempPath.setText(pystring(self.postgisQueryTmpPath))     
     pass
      
#@brief Executing the Query via pgsql2shp and adding the resulting shape to map canvas
#@param string query
#@return  none
#@author Dr. Horst Duester
#@date 10. March 2009
#@version 1.0
#@todo
  def showVLayer(self, query):
     QApplication.setOverrideCursor(Qt.WaitCursor)
     tmpLayerDef = self.createShapeFileName(self.postgisQueryTmpPath)
     file = tmpLayerDef[0]
     layerName = tmpLayerDef[1]

#  PG Serverdefinintion holen
     
     myServer = self.dlg.cmbServer.currentText()
     if len(myServer) > 0:
         settings = QSettings()
         mySettings = "/PostgreSQL/connections/"+myServer
         dbName = settings.value(pystring(mySettings+"/database"))
         dbUser = settings.value(pystring(mySettings+"/username"))
         dbHost = settings.value(pystring(mySettings+"/host"))
         dbPort = settings.value(pystring(mySettings+"/port"))
         dbPasswd = settings.value(pystring(mySettings+"/password"))
         query = query.replace('"','\\"')
    
         cmdHost = ""
         cmdUser = ""
         cmdPasswd = ""
    
         if len(dbHost) != 0:
           cmdHost = "-h "+dbHost
         if len(dbUser) != 0:
           cmdUser = "-u "+dbUser
         if len(dbPasswd) != 0:
           cmdPasswd = "-P "+dbPasswd
         
    
         osCmd = pystring("pgsql2shp -f \""+str(file)+"\" "+cmdHost+" "+cmdUser+" -p "+str(dbPort)+" "+cmdPasswd+" "+str(dbName)+" \""+query+"\"")
    #     QMessageBox.information(None,'',osCmd)
         proc = subprocess.Popen(osCmd.encode('latin1'),
                               shell=True,
                               stdin=subprocess.PIPE,
                               stdout=subprocess.PIPE,
                               stderr=subprocess.PIPE
                               )
         stdout_value, stderr_value = proc.communicate()
         self.dlg.txtOutput.setText(stdout_value+stderr_value)
           
         uri = file
         vLayer = QgsVectorLayer(uri,  layerName,  "ogr")
         canvas = self.iface.mapCanvas()
         rect = canvas.extent()
         
         try:
             QgsMapLayerRegistry.instance().addMapLayer(vLayer)         
         except:
             QgsMapLayerRegistry.instance().addMapLayers([vLayer])
     else:
         QMessageBox.information(None, 'Error',  'Please select a database connection')
     QApplication.restoreOverrideCursor()
     QApplication.setOverrideCursor(Qt.ArrowCursor)  
     pass
    
#@brief define the temporary shape file name
#@param String path to the temporary folder
#@return  tmpShapeName
#@author Dr. Horst Duester
#@date 10. March 2009
#@version 1.0
#@todo
  def createShapeFileName(self, myPath):
     if myPath[-1] == "/": 
       myPath = myPath
     else:
       myPath = myPath + "/"     

     i  = 1 
     myTempShape = str(myPath) + "querytmp" + str(i) + ".shp"
     myTempLayer = "querytmp" + str(i)
     
     while os.path.isfile(pystring(myTempShape)):
        i  = i + 1
        myTempShape = str(myPath) + "querytmp" + str(i) + ".shp"
        myTempLayer = "querytmp" + str(i)
        
     myLayer = []   
     myLayer.append(myTempShape)
     myLayer.append(myTempLayer)
     
     return myLayer
     
#@brief loading a query from file
#@param none
#@return  Void
#@author Dr. Horst Duester
#@date 10. March 2009
#@version 1.0
#@todo     
  def loadSqlFile(self):
    fileName = QFileDialog.getOpenFileName(None, "Open SQL-File","",  "SQL File (*.sql);;All Files (*.*)")
    myFile = QFile(fileName)
    if (not myFile.open(QIODevice.ReadOnly | QIODevice.Text)):
      pass
      
    myText = QTextStream(myFile)
    mySql = ""
    mySql = myText.readAll()
    self.dlg.txtEditSql.setText(mySql)
    
    
#@brief  saving query t file
#@param none
#@return  Void
#@author Dr. Horst Duester
#@date 10. March 2009
#@version 1.0
#@todo     
  def saveSqlFile(self):
    fileNameArr = QFileDialog.getSaveFileName(None, "Save SQL-File","",  "SQL File (*.sql);; All Files (*.*)").split('.')

    if len(fileNameArr)==1:
      fileName = fileNameArr[0]+'.sql'
    else:
      fileName = fileNameArr[0]+'.'+fileNameArr[1]

    myFile = QFile(fileName)
    
    
    if (not myFile.open(QIODevice.ReadOnly | QIODevice.Text)):
      pass
      
    myText = QTextStream(myFile)
    mySql = pystring(self.dlg.txtEditSql.toPlainText().trimmed())
    
    import codecs
    s = codecs.open(fileName, 'w','LATIN1')
    s.write(mySql)
    s.close()
